# -*- coding: utf-8 -*-

from datetime import datetime
import urlparse

from django.db import models
from django.db.models import permalink
from django.core.urlresolvers import reverse


class SimpleURLField(CharField):
	def pre_save(self, model_instance, add):
		url=getattr(model_instance, self.attname, '')
		if add and url and not url.startswith('http://'):
			url='http://%s'%url
			# url=urlparse.urlunparse(urlparse.urlparse(url,'http'))
			setattr(model_instance, self.attname, url)
			return url
		return CharField.pre_save(self, model_instance, add)


class TreeOrderField(models.CharField):
	def pre_save(self, model_instance, add):
		if add:
			parent=(model_instance.parent or model_instance.article);
			if parent.seq<1000:
				parent.seq+=1;
				parent.save();
			else:
				# update all comments to use 4digit masks
				pass
			value=('%s%03d'%(getattr(parent, self.attname, ''), parent.seq, ))[:255]
			setattr(model_instance, self.attname, value)
			return value
		return models.CharField.pre_save(self, model_instance, add)


class Comment(models.Model):
	article=models.ForeignKey(Article)
	parent=models.ForeignKey('self', blank=True, null=True, related_name='child_set')
	author_name=models.CharField(max_length=32)
	author_email=models.EmailField()
	# author_url=models.URLField(blank=True)
	author_url=models.SimpleURLField(max_length=200, blank=True)
	text=models.TextField()
	pub_date=models.DateTimeField('date published', default=datetime.now)
	admin_comment=models.BooleanField(default=False)
	seq=models.PositiveSmallIntegerField(default=0)
	path=TreeOrderField(max_length=255, blank=True)

	@property
	def level(self):
		return max(0,len(self.path)/3-1)

	class Meta:
		ordering=[ 'pub_date', ]

	class Admin:
		list_display=('pub_date', 'author_name', 'author_url', 'author_email', )
		search_fields=('text', )
		list_filter=('author_name', )
		date_hierarchy='pub_date'
		fields=(
			('Author', {'fields':('author_name', 'author_email', 'author_url', )}),
			('Comment', {'fields':('text', 'admin_comment', 'pub_date', )}),
			('Advanced', {'fields':('article', 'parent', 'seq', 'path', ), 'classes': 'collapse'}),
		)

	def __unicode__(self):
		return self.author_name

	# @permalink
	def get_absolute_url(self):
		return '%(article)s#cm%(cm_id)d'%{
			'article': reverse('article_view', None, { 'article_slug': self.article.slug, }),
			'cm_id': self.id,
		}

	def author_favicon(self):
		if not self.author_url:
			return ''
		url=urlparse.urlparse(self.author_url,'http')
		favicon=urlparse.urlunparse(('http', url[1], '/favicon.ico', '', '', ''))
		return favicon
